@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                    Copyright (C) 2007 QRW Software
+--                 Copyright (C) 2007-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,14 @@
 --
 --
 -- $Log: acronym.c,v $
+-- Revision 1.5  2008/09/01 00:10:48  jsedwards
+-- Fix copyright year.  NO code changes.
+--
+-- Revision 1.4  2008/08/31 21:53:51  jsedwards
+-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
+-- and nwos_read_object_from_disk because now they return false when they fail
+-- instead of asserting themselves.
+--
 -- Revision 1.3  2007/07/01 19:56:15  jsedwards
 -- Remove spurious line accidentally left in.
 --
@@ -162,9 +170,9 @@ bool nwos_acronym_to_string(ObjRef* ref, char* string, size_t size)
     C_struct_Acronym acronym_obj;
     int j;
 
-    nwos_read_object_from_disk(ref, &acronym_obj, sizeof(acronym_obj));
+    assert(nwos_read_object_from_disk(ref, &acronym_obj, sizeof(acronym_obj)));
 
-    nwos_read_variable_sized_object_from_disk(&acronym_obj.spelling, kludge, sizeof(kludge), &get_spelling_object_size);
+    assert(nwos_read_variable_sized_object_from_disk(&acronym_obj.spelling, kludge, sizeof(kludge), &get_spelling_object_size));
 
     if (size < ptr_spelling_obj->count + 1) return false;
 
