@@ -27,6 +27,22 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.93  2008/09/01 18:08:19  jsedwards
+-- Added chunk_info_reverse_index to get from chunk number back to an index
+-- into the chunk_info table.
+--
+-- Revision 1.92  2008/09/01 16:18:10  jsedwards
+-- Removed nwos_allocate_all_chunks_hack, no longer used.  Removing the
+-- --allocate-all option from prep_disk.
+--
+-- Revision 1.91  2008/09/01 15:44:01  jsedwards
+-- Fix bug in nwos_allocate_new_chunk where it wasn't searching the chunk_info
+-- table to insert new chunk.
+--
+-- Revision 1.90  2008/09/01 02:57:40  jsedwards
+-- Fix so that if hash isn't found or if block isn't in compressed index
+-- nwos_read_block returns false.
+--
 -- Revision 1.89  2008/08/10 15:16:38  jsedwards
 -- Added printing of path in "Missing magic number" error message.
 --
@@ -404,6 +420,7 @@ static ObjRef* storage_index;
 static int     storage_count;
 
 static Chunk_Info* chunk_info;
+static int* chunk_info_reverse_index;  /* index from chunk index to info table */
 static bool chunk_info_modified;
 static uint16* chunk_used_index;  /* order of chunks sorted by blocks used, fewest to most used */
 
@@ -659,6 +676,43 @@ void initialize_random_number_generator()
 #endif
 }
 
+/*************************************************************************************/
+/* This updates the chunk_info_reverse_index table to point to the chunk_info table. */
+/*                                                                                   */
+/*  chunk_info   reverse                                                             */
+/*    0 [5]       0 [4]                                                              */
+/*    1 [3]       1 [5]                                                              */
+/*    2 [4]       2 [3]                                                              */
+/*    3 [2]       3 [1]                                                              */
+/*    4 [0]       4 [2]                                                              */
+/*    5 [1]       5 [0]                                                              */
+/*                                                                                   */
+/*************************************************************************************/
+
+static void update_chunk_info_reverse_index()
+{
+    int i;
+
+    assert(nwos_total_private_chunks > 0);
+
+    if (chunk_info_reverse_index == NULL)
+    {
+	chunk_info_reverse_index = nwos_malloc(nwos_total_private_chunks * sizeof(int));
+    }
+
+    for (i = 0; i < nwos_used_private_chunks; i++)
+    {
+	assert(chunk_info[i].index < nwos_used_private_chunks);
+	chunk_info_reverse_index[chunk_info[i].index] = i;
+    }
+
+    /* verify it is correct */
+    for (i = 0; i < nwos_used_private_chunks; i++)
+    {
+	assert(chunk_info[chunk_info_reverse_index[i]].index == i);
+    }
+}
+
 
 void nwos_initialize_disk_io(AccessType type, const char* path)
 {
@@ -887,6 +941,9 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
 	chunk_info_modified = false;
 
+	update_chunk_info_reverse_index();
+
+
 	/* now create the chunk_used_index */
 
 	chunk_info_size_in_bytes = nwos_total_private_chunks * sizeof(uint16);
@@ -1124,14 +1181,15 @@ void nwos_allocate_new_chunk(uint32 ref)
     assert(nwos_hash_uint32_ref(ref) == 0);
 
     /* figure out where this ref goes in the chunk_info table */
-    for (i = nwos_used_private_chunks; i > 0; i--)
+    for (i = nwos_used_private_chunks - 1; i >= 0; i--)
     {
 	if (chunk_info[i].ref < ref) break;
 
-	chunk_info[i].ref = chunk_info[i-1].ref;
-	chunk_info[i].used = chunk_info[i-1].used;
-	chunk_info[i].index = chunk_info[i-1].index;
+	chunk_info[i+1].ref = chunk_info[i].ref;
+	chunk_info[i+1].used = chunk_info[i].used;
+	chunk_info[i+1].index = chunk_info[i].index;
     }
+    i++;
 
     offset = (ref - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
 
@@ -1185,75 +1243,7 @@ void nwos_allocate_new_chunk(uint32 ref)
 
     nwos_used_private_chunks += 1;
 
-    chunk_info_modified = true;
-}
-
-
-void nwos_allocate_all_chunks_hack(uint32 ref, void (*progress_report)(float))
-{
-    uint32 offset;
-    static uint8* chunk;
-    uint32 normalized_ref;
-
-    assert(RESERVED_PUBLIC_BLOCKS <= ref && ref <= MAXIMUM_VALID_PRIVATE_REFERENCE);
-
-    assert(nwos_hash_uint32_ref(ref) == 0);
-
-    assert(nwos_used_private_chunks == 0);
-
-    chunk = malloc(CHUNK_SIZE);
-
-    if (chunk == NULL)
-    {
-	perror("allocate memory for creating new chunk");
-	exit(1);
-    }
-
-    memset(chunk, 0, CHUNK_SIZE);
-
-    chunk[0] = 0xff;
-    chunk[1] = 0xff;
-    chunk[2] = 0xff;
-    chunk[3] = 0xff;
-
-    normalized_ref = ((ref - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK) * USABLE_BLOCKS_PER_CHUNK + RESERVED_PUBLIC_BLOCKS;
-
-    while (nwos_used_private_chunks < nwos_total_private_chunks)
-    {
-	if (progress_report != NULL)
-	{
-	    (*progress_report)((float)nwos_used_private_chunks / (float)nwos_total_private_chunks);
-	}
-
-	chunk_info[nwos_used_private_chunks].ref = normalized_ref;
-	chunk_info[nwos_used_private_chunks].index = nwos_used_private_chunks;
-
-	offset = nwos_block_offset_to_chunks + (nwos_used_private_chunks * BLOCKS_IN_CHUNK);
-
-	if (lseek(private_file_desc, (off_t)offset << 8, SEEK_SET) < 0)
-	{
-	    perror(nwos_private_path);
-	    exit(1);
-	}
-
-	if (write(private_file_desc, chunk, BIT_MAP_BYTES) != BIT_MAP_BYTES)
-	{
-	    perror(nwos_private_path);
-	    exit(1);
-	}
-
-	if (fdatasync(private_file_desc) != 0)
-	{
-	    perror(nwos_private_path);
-	    exit(1);
-	}
-
-	usleep(10000);
-
-	normalized_ref += USABLE_BLOCKS_PER_CHUNK;
-
-	nwos_used_private_chunks += 1;
-    }
+    update_chunk_info_reverse_index();
 
     chunk_info_modified = true;
 }
@@ -1389,9 +1379,7 @@ static inline off_t ref_to_offset(ObjRef* ref)
 
     hash = nwos_hash_ref(ref);
 
-    assert(hash != 0);
-
-    return (off_t) hash << 8;
+    return hash == 0 ? (off_t) -1 : (off_t) hash << 8;
 }
 
 
@@ -1547,45 +1535,9 @@ void read_bit_map_into_cache(Bit_Map_Cache_Entry* entry)
 
 static int chunk_index_to_info_index(uint16 chunk_index)
 {
-    int lwr;
-    int upr;
-    int mid = 0;
+    assert(chunk_index < nwos_used_private_chunks);
 
-    assert(nwos_used_private_chunks > 0);
-    assert(chunk_index < nwos_total_private_chunks);
-
-    lwr = 1;
-
-    upr = nwos_used_private_chunks;
-
-    while (lwr <= upr)
-    {
-	mid = (upr + lwr) / 2;
-
-	if (chunk_index < chunk_info[mid - 1].index)
-	{
-	    upr = mid - 1;
-	}
-	else if (chunk_index > chunk_info[mid - 1].index)
-	{
-	    lwr = mid + 1;
-	}
-	else
-	{
-	    mid = mid - 1;  /* adjust to 0 based index */
-	    break;
-	}
-    }
-
-    if (lwr <= upr)
-    {
-	assert(0 <= mid && mid < nwos_used_private_chunks);
-	assert(chunk_info[mid].index == chunk_index);
-
-	return mid;
-    }
-
-    return -1;   /* not found */
+    return chunk_info_reverse_index[chunk_index];
 }
 
 
@@ -2488,14 +2440,18 @@ bool nwos_read_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
 		fprintf(stderr, "  %d: %08x\n", i, nwos_ref_to_word(&storage_index[i]));
 	    }
 
-	    exit(1);
+	    offset = (off_t) -1;
 	}
+	else
+	{
+	    assert(is_same_object(&storage_index[i], ref));
 
-	assert(is_same_object(&storage_index[i], ref));
-
-	offset = (off_t)i * FILE_BLOCK_SIZE;
+	    offset = (off_t)i * FILE_BLOCK_SIZE;
+	}
     }
 
+    if (offset < 0) return false;
+
     if (lseek(file_desc, offset, SEEK_SET) < 0)
     {
 	uint32 offset_upr = (uint32)(offset >> 32);
@@ -2564,6 +2520,9 @@ void nwos_write_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
     assert(storage_index == NULL);   /* at this time if there is a storage index it is read only */
 
     offset = ref_to_offset(ref);
+
+    assert(offset >= 0);
+
 #ifdef PUBLIC_MODE
     if (lseek(public_file_desc, offset, SEEK_SET) < 0)
 #else
@@ -2620,6 +2579,9 @@ void nwos_remove_object(ObjRef* ref)
     assert(sizeof(block) == FILE_BLOCK_SIZE);
 
     offset = ref_to_offset(ref);
+
+    assert(offset >= 0);
+
     if (lseek(private_file_desc, offset, SEEK_SET) < 0)
     {
 	uint32 offset_upr = (uint32)(offset >> 32);
