@@ -27,6 +27,12 @@
 --
 --
 -- $Log: expand_sparse.c,v $
+-- Revision 1.33  2008/09/01 03:15:46  jsedwards
+-- Change to call nwos_initialize_disk_io instead of nwos_initialize_objectify.
+--
+-- Revision 1.32  2008/08/30 12:56:13  jsedwards
+-- Changed parameters for nwos_initialize_objectify call.
+--
 -- Revision 1.31  2008/07/19 13:15:33  jsedwards
 -- If 0'd out printing of every block's reference ID so it doesn't spew every
 -- block ID to the screen anymore.  Added calls to print a progress bar instead.
@@ -227,7 +233,7 @@ int main(int argc, char* argv[])
 
     obj_file_path = nwos_get_private_objects_path();
 
-    nwos_initialize_objectify(NULL, 0, 0, READ_WRITE, DEFAULT_FILE);
+    nwos_initialize_disk_io(READ_WRITE, DEFAULT_FILE);
 
     nwos_terminate_backup(false, NULL);   /* don't create a backup file */
 
