@@ -27,6 +27,23 @@
 --
 --
 -- $Log: export_file.c,v $
+-- Revision 1.23  2008/09/01 03:13:42  jsedwards
+-- Change for new nwos_initialize_objectify calling convention (doesn't pass
+-- back root_object_reference anymore) and removed call to nwos_set_root_object
+-- because initialize calls it now.
+--
+-- Revision 1.22  2008/08/31 21:53:52  jsedwards
+-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
+-- and nwos_read_object_from_disk because now they return false when they fail
+-- instead of asserting themselves.
+--
+-- Revision 1.21  2008/08/30 12:46:10  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
+-- Revision 1.20  2008/06/05 11:04:14  jsedwards
+-- Fixed to find the latest version of the path and file association object.
+--
 -- Revision 1.19  2008/03/09 15:41:12  jsedwards
 -- Added code to create subdirectories if they don't exist.  (File changed on
 -- laptop on 2008/02/29 in Columbia, South Carolina but could not check in
@@ -143,6 +160,7 @@ void export_file(char* name)
     ObjRef assoc_ref;
     C_struct_File file_obj;
     C_struct_Path_And_File_Association assoc_obj;
+    C_struct_Path_And_File_Association next_assoc_obj;
     int num_files;
     char* p;
 
@@ -163,9 +181,23 @@ void export_file(char* name)
 	{
 	    assert(nwos_file_path_to_path_and_file_association(&path_ref, 0, &assoc_ref));
 
-	    nwos_read_object_from_disk(&assoc_ref, &assoc_obj, sizeof(assoc_obj));  /* read the assoc. object */
+	    assert(nwos_read_object_from_disk(&assoc_ref, &assoc_obj, sizeof(assoc_obj)));  /* read the assoc. object */
 
-	    nwos_read_object_from_disk(&assoc_obj.file, &file_obj, sizeof(file_obj));  /* read the file object */
+	    while (!is_void_reference(&assoc_obj.header.object.next_version))
+	    {
+		assert(nwos_read_object_from_disk(&assoc_obj.header.object.next_version, &next_assoc_obj, sizeof(next_assoc_obj)));
+
+		if (!is_same_object(&next_assoc_obj.path, &path_ref))   /* it has a different name */
+		{
+		    assert(is_same_object(&next_assoc_obj.file, &assoc_obj.file));
+		    break;
+		}
+
+		copy_reference(&assoc_ref, &assoc_obj.header.object.next_version);
+		memcpy(&assoc_obj, &next_assoc_obj, sizeof(assoc_obj));
+	    }
+
+	    assert(nwos_read_object_from_disk(&assoc_obj.file, &file_obj, sizeof(file_obj)));  /* read the file object */
 
 	    if (is_void_reference(&file_obj.block_list))
 	    {
@@ -208,17 +240,13 @@ void export_file(char* name)
 
 int main(int argc, char* argv[])
 {
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
-    ObjRef root_object_ref;
     ObjRef path_class_ref;
     ObjRef object_class;
     ObjRef path_ref;
     ObjRef assoc_ref;
     C_struct_Class_Definition class_def_obj;
     C_struct_Path_And_File_Association assoc_obj;
+    C_struct_Path_And_File_Association next_assoc_obj;
     uint8 kludge[FILE_BLOCK_SIZE];
     C_struct_File_Path* ptr_path_obj = (C_struct_File_Path*)kludge;
     int argn;
@@ -253,16 +281,7 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
-    nwos_get_key_from_password(big_key, sizeof(big_key));
-
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-    nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, path);
-
-    nwos_set_root_object(&root_object_ref);
+    nwos_initialize_objectify(READ_ONLY, path);
 
 
     if (!nwos_find_private_class_definition("FILE PATH", &path_class_ref))
@@ -289,7 +308,7 @@ int main(int argc, char* argv[])
 
 	    if (is_same_object(&object_class, &path_class_ref))
 	    {
-		nwos_read_variable_sized_object_from_disk(&ref_list->references[j], kludge, sizeof(kludge), &get_path_object_size);
+		assert(nwos_read_variable_sized_object_from_disk(&ref_list->references[j], kludge, sizeof(kludge), &get_path_object_size));
 
 		/* remember ptr_path_obj points to the kludge buffer */
 
@@ -311,9 +330,23 @@ int main(int argc, char* argv[])
 			assert(nwos_find_file_path(name, &path_ref));
 			assert(nwos_file_path_to_path_and_file_association(&path_ref, 0, &assoc_ref));
 
-			nwos_read_object_from_disk(&assoc_ref, &assoc_obj, sizeof(assoc_obj));  /* read the assoc. object */
+			assert(nwos_read_object_from_disk(&assoc_ref, &assoc_obj, sizeof(assoc_obj)));  /* read the assoc. object */
 
-			if (is_void_reference(&assoc_obj.header.object.next_version))  /* only match wildcards to names that aren't old (renamed files) */
+			while (!is_void_reference(&assoc_obj.header.object.next_version))  /* find latest version */
+			{
+			    assert(nwos_read_object_from_disk(&assoc_obj.header.object.next_version, &next_assoc_obj, sizeof(next_assoc_obj)));
+
+			    if (!is_same_object(&next_assoc_obj.path, &path_ref))   /* it has a different name */
+			    {
+				assert(is_same_object(&next_assoc_obj.file, &assoc_obj.file));
+				break;    /* don't match old file names to wildcards */
+			    }
+
+			    copy_reference(&assoc_ref, &assoc_obj.header.object.next_version);
+			    memcpy(&assoc_obj, &next_assoc_obj, sizeof(assoc_obj));
+			}
+
+			if (is_void_reference(&assoc_obj.header.object.next_version))
 			{
 			    export_file(name);
 			    num_files++;
