@@ -27,6 +27,82 @@
 --
 --
 -- $Log: objectify.c,v $
+-- Revision 1.175  2008/09/01 18:22:17  jsedwards
+-- Moved most of the code from add_password.c and prep_disk.c that prepared
+-- to create the root object into nwos_create_root.  Also fixed it to read
+-- the password twice and verify they are the same before creating it.
+--
+-- Revision 1.174  2008/09/01 12:50:14  jsedwards
+-- Extracted new function "setup_from_big_key" from nwos_initialize_objectify
+-- to create the sequence table from the big_key.
+--
+-- Revision 1.173  2008/09/01 03:06:06  jsedwards
+-- Changed so that nwos_initialize_objectify calls nwos_set_root and allows
+-- up to three tries for entering the correct password.
+--
+-- Revision 1.172  2008/08/31 18:00:14  jsedwards
+-- Add assert around call to nwos_read_object_from_disk because it now returns
+-- false when it fails, whereas before it would assert itself.
+--
+-- Revision 1.171  2008/08/31 10:49:57  jsedwards
+-- Changed nwos_read_variable_sized_object_from_disk to return bool (success
+-- or failure) instead of asserting on failure.
+--
+-- Revision 1.170  2008/08/31 10:38:04  jsedwards
+-- Change nwos_read_object_from_disk to return success or failure instead of
+-- asserting when a failure occurs.
+--
+-- Revision 1.169  2008/08/31 10:19:17  jsedwards
+-- Change check_object_data to return true or false instead of exiting.
+--
+-- Revision 1.168  2008/08/30 12:53:53  jsedwards
+-- Corrected previous log entry (accidentally checked in with the incorrect
+-- message).  NO code changes.
+--
+-- Revision 1.167  2008/08/30 12:46:15  jsedwards
+-- Changed nwos_initialize_objectify to get the pass phrase instead of
+-- passing in the key parameters.  It now returns the private root object
+-- reference.
+--
+-- Revision 1.166  2008/08/01 13:42:20  jsedwards
+-- Added setting header to point to object back in read object from disk
+-- functions, so if object is larger than one block it reads the correct data
+-- checksum.  It was changed to point to extra block when checking for old
+-- encryption in previous check in.
+--
+-- Revision 1.165  2008/07/31 14:40:25  jsedwards
+-- Changed to try old decryption algorithm if checksum fails after reading
+-- with new decryption algorithm.
+--
+-- Revision 1.164  2008/07/31 14:16:52  jsedwards
+-- Fixed bug in check_object_header where it was or'ing together the checksum
+-- comparison instead of and'ing them, so if any one byte matched it considered
+-- it correct, instead of testing each and every byte.
+--
+-- Revision 1.163  2008/07/31 11:19:57  jsedwards
+-- Changed the order of the blowfish encryption and the reordering of the bytes
+-- to make it harder to decrypt the data.  Originally it reordered the bytes
+-- first and then applied the encryption.  The problem with that was that if
+-- the encryption was broken it would be more trivial to reorder the data
+-- because the data itself would give hints as to the correct position of the
+-- data (zeros could be picked out for example).  Now the encryption is applied
+-- first and then the data is scrambled.  This way the descrambling has to be
+-- done first while the data is hidden, when there aren't any clues as to it's
+-- position.
+--
+-- Revision 1.162  2008/07/30 14:19:21  jsedwards
+-- Eliminated 'ptrobj' variable in encrypt and decrypt functions, changed to
+-- use memcpy instead of for loop, and eliminated copying the first 8 bytes
+-- twice.
+--
+-- Revision 1.161  2008/07/30 14:06:02  jsedwards
+-- Changed 'buffer1' and 'buffer2' in the decrypt and encrypt functions to
+-- 'scrambled' and 'encrypted' respectively, to make it easier to follow the
+-- flow of the data.
+--
+-- Revision 1.160  2008/07/30 13:48:19  jsedwards
+-- Removed old DEBUG_ENCRYPTION code, it is just cluttering up the place.
+--
 -- Revision 1.159  2008/07/19 14:40:55  jsedwards
 -- Added a hack to allow the root object to have a public class definition and
 -- write_object_to_disk to not assert.
@@ -1058,15 +1134,57 @@ static void generate_sequence_table(uint32* linear, uint32* serial, uint8 table[
 /* Initialization and Termination */
 /**********************************/
 
-void nwos_initialize_objectify(uint8 bf_key[16], uint32 linear_seed, uint32 serial_seed, AccessType type, const char* path)
+static void setup_from_big_key(uint8 big_key[16 + 8 + 4], ObjRef* root_object_ref)
 {
-#ifdef PUBLIC_MODE
-    assert(type == PUBLIC);
-#else
     int i;
     int j;
     int k;
     int tries;
+
+    BF_set_key(&blowfish_key, 16, big_key);
+
+    linear_after_first = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
+    serial_after_first = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
+
+    /* Create all of the sequence tables from the linear and serial parameters */
+    for (i = 0; i < NUM_STORED_SEQ; i++)
+    {
+	tries = 0;
+
+	do
+	{
+	    tries++;
+
+	    assert(tries < 4);
+
+	    generate_sequence_table(&linear_after_first, &serial_after_first, nwos_random_sequence[i]);
+
+	    for (j = 0; j < i; j++)
+	    {
+		for (k = 8; k < FILE_BLOCK_SIZE; k++) if (nwos_random_sequence[i][k] != nwos_random_sequence[j][k]) break;
+
+		if (k == FILE_BLOCK_SIZE) break;
+	    }
+	}
+	while (j < i);   /* if j is less than i it means one of the tables matched and needs to be regenerated */
+    }
+
+    memcpy(root_object_ref->id, big_key+20, 4);
+}
+
+
+void nwos_initialize_objectify(AccessType type, const char* path)
+{
+#ifdef PUBLIC_MODE
+    assert(type == PUBLIC);
+#else
+    int attempt;  /* number of pass phrase attempts */
+    uint8 big_key[16 + 8 + 4];
+    ObjRef root_object_ref;
+    char* incorrect_messages[] = { "Incorrect, try again.", "Wrong again, one more try.", "Sorry, no go." };
+
+#define NUM_ATTEMPTS (sizeof(incorrect_messages) / sizeof (char*))
+
 #endif
 
     nwos_log("nwos_initialize_objectify():");
@@ -1074,144 +1192,186 @@ void nwos_initialize_objectify(uint8 bf_key[16], uint32 linear_seed, uint32 seri
     nwos_crc32_initialize();
     nwos_initialize_disk_io(type, path);
 
-    /* are we starting up with security or not? */
-
     if (type == PUBLIC)
     {
-	assert(bf_key == NULL && linear_seed == 0 && serial_seed == 0);
+	root_object_ref.id[0] = 0;
+	root_object_ref.id[1] = 0;
+	root_object_ref.id[2] = 0;
+	root_object_ref.id[3] = 1;
+
+	assert(nwos_set_root_object(&root_object_ref));
     }
 #ifndef PUBLIC_MODE
     else
     {
-	if (bf_key != NULL)
+	for (attempt = 0; attempt < NUM_ATTEMPTS; attempt++)
 	{
-	    linear_after_first = linear_seed;
-	    serial_after_first = serial_seed;
-#if 0
-	    int i;
-	    printf("blowfish key: ");
-	    for (i = 0; i < 16; i++) printf("%02x", bf_key[i]);
-	    printf("\n");
-#endif
-	    BF_set_key(&blowfish_key, 16, bf_key);
+	    nwos_get_key_from_password(big_key, sizeof(big_key));
 
-	    /* Create all of the sequence tables from the linear and serial parameters */
-	    for (i = 0; i < NUM_STORED_SEQ; i++)
-	    {
-		tries = 0;
+	    setup_from_big_key(big_key, &root_object_ref);
 
-		do
-		{
-		    tries++;
-
-		    assert(tries < 4);
-
-		    generate_sequence_table(&linear_after_first, &serial_after_first, nwos_random_sequence[i]);
-
-		    for (j = 0; j < i; j++)
-		    {
-			for (k = 8; k < FILE_BLOCK_SIZE; k++) if (nwos_random_sequence[i][k] != nwos_random_sequence[j][k]) break;
-
-			if (k == FILE_BLOCK_SIZE) break;
-		    }
-		}
-		while (j < i);   /* if j is less than i it means one of the tables matched and needs to be regenerated */
-	    }
+	    if (nwos_set_root_object(&root_object_ref)) break;   /* we have a winner! */
+	    
+	    fprintf(stderr, "%s\n", incorrect_messages[attempt]);
 	}
-	else   /* if the blowfish key is null, all of the values should be null */
+
+	if (attempt == NUM_ATTEMPTS)
 	{
-	    assert(bf_key == NULL && linear_seed == 0 && serial_seed == 0);
+	    nwos_terminate_disk_io();
+	    fprintf(stderr, "\n");
+	    exit(1);
 	}
     }
 #endif
 }
 
 
-/* returns true if the object was created, false if it already existed */
-
 static bool creating_root = false;
 
-bool nwos_create_root(ObjRef* ref)
+/* this is the same as the normal routine in objectify.c except it doesn't translate the reference for the disk */
+static void set_public_root_object(ObjRef* ref)
 {
     C_struct_Root root_obj;
-    ObjRef root_class_ref;
-    bool result = false;
+    ObjRef verify_ref;
 
-    /* first find out if we already have this root */
+    assert(nwos_read_object_from_disk(ref, &root_obj, sizeof(root_obj)));
+    copy_reference(&nwos_public_class_definition_class_ref, &root_obj.class_definition_class);
+    copy_reference(&nwos_reference_list_class_ref, &root_obj.reference_list_class);
 
-#ifdef PUBLIC_MODE
-    if (!nwos_object_exists(ref))   /* not already used */
-    {
-#else
-    uint32 hash = nwos_hash_ref(ref);
-
-    if (!nwos_test_bit_in_map(hash))   /* not already used */
-    {
-	nwos_set_bit_in_map(hash);
-#endif
-	/* currently I can't think of any reason to have a private root class, since there is no list of root objects */
-	assert(nwos_find_public_class_definition("ROOT", &root_class_ref));
-
-	memset(&root_obj, 0, sizeof(root_obj));  /* zero it out */
-
-	nwos_fill_in_common_header(&root_obj.header.common, ref, &root_class_ref);
-#ifdef PUBLIC_MODE
-	memcpy(&root_obj.class_definition_class, &nwos_public_class_definition_class_ref, sizeof(root_obj.class_definition_class));
-#else
-	memcpy(&root_obj.class_definition_class, &nwos_private_class_definition_class_ref, sizeof(root_obj.class_definition_class));
-#endif
-	memcpy(&root_obj.reference_list_class, &nwos_reference_list_class_ref, sizeof(root_obj.reference_list_class));
-
-	nwos_crc32_calculate((uint8*) &root_obj.header.object, sizeof(ObjectHeader), root_obj.header.common.header_chksum);
-
-	nwos_crc32_calculate((uint8*) &root_obj.class_definition_class, sizeof(root_obj) - sizeof(EveryObject), root_obj.header.common.data_chksum);
-
-	creating_root = true;
-	nwos_write_object_to_disk(ref, &root_obj, sizeof(root_obj));
-	creating_root = false;
-
-#ifdef PUBLIC_MODE
-	nwos_add_to_references(ref, &root_class_ref);
-#endif
-
-	result = true;
-    }
-
-    return result;
+    assert(nwos_find_public_class_definition("REFERENCE LIST", &verify_ref));
+    assert(is_same_object(&verify_ref, &nwos_reference_list_class_ref));
 }
 
+    
+#ifndef PUBLIC_MODE
+void nwos_create_root(void)
+{
+    uint8 big_key_1[16 + 8 + 4];
+    uint8 big_key_2[16 + 8 + 4];
+    uint32 hash;
+    ObjRef public_root_obj_ref;
+    ObjRef private_root_obj_ref;
+    ObjRef public_root_class_ref;
+    C_struct_Root root_obj;
+    ObjRef root_class_ref;
 
-void nwos_set_root_object(ObjRef* ref)
+
+    nwos_log("nwos_create_root():");
+
+    nwos_crc32_initialize();
+    nwos_initialize_disk_io(READ_WRITE, DEFAULT_FILE);
+
+    assert(nwos_total_private_chunks > nwos_used_private_chunks);
+
+    public_root_obj_ref.id[0] = 0;
+    public_root_obj_ref.id[1] = 0;
+    public_root_obj_ref.id[2] = 0;
+    public_root_obj_ref.id[3] = 1;
+
+    set_public_root_object(&public_root_obj_ref);
+
+    assert(nwos_find_public_class_definition("ROOT", &public_root_class_ref));
+
+
+    while (true)
+    {
+	assert(sizeof(big_key_1) == sizeof(big_key_2));
+
+	nwos_get_key_from_password(big_key_1, sizeof(big_key_1));
+
+	setup_from_big_key(big_key_1, &private_root_obj_ref);
+
+	hash = nwos_hash_ref(&private_root_obj_ref);
+
+	if (hash != 0 && nwos_test_bit_in_map(hash))   /* that block is already used */
+	{
+	    fprintf(stderr, "\nThat pass phrase cannot be used, try a different one.\n\n");
+	}
+	else
+	{
+	    nwos_get_key_from_password(big_key_2, sizeof(big_key_2));
+
+	    if (memcmp(big_key_1, big_key_2, sizeof(big_key_1)) == 0) break;
+
+	    fprintf(stderr, "\nKeys don't match, try again\n\n");
+	}
+    }
+
+    /* create the private root object */
+
+    nwos_allocate_new_chunk(nwos_ref_to_word(&private_root_obj_ref));
+
+    hash = nwos_hash_ref(&private_root_obj_ref);
+
+    assert(hash != 0 && !nwos_test_bit_in_map(hash));    /* make sure chunk is allocated, but block is not already used */
+
+    nwos_set_bit_in_map(hash);
+
+    nwos_set_security_level(Security_Extreme);
+
+    nwos_clone_class_definition(&nwos_public_class_definition_class_ref, &nwos_private_class_definition_class_ref);
+
+
+    /* currently I can't think of any reason to have a private root class, since there is no list of root objects */
+    assert(nwos_find_public_class_definition("ROOT", &root_class_ref));
+
+    memset(&root_obj, 0, sizeof(root_obj));  /* zero it out */
+
+    nwos_fill_in_common_header(&root_obj.header.common, &private_root_obj_ref, &root_class_ref);
+
+    memcpy(&root_obj.class_definition_class, &nwos_private_class_definition_class_ref, sizeof(root_obj.class_definition_class));
+
+    memcpy(&root_obj.reference_list_class, &nwos_reference_list_class_ref, sizeof(root_obj.reference_list_class));
+
+    nwos_crc32_calculate((uint8*) &root_obj.header.object, sizeof(ObjectHeader), root_obj.header.common.header_chksum);
+
+    nwos_crc32_calculate((uint8*) &root_obj.class_definition_class, sizeof(root_obj) - sizeof(EveryObject), root_obj.header.common.data_chksum);
+
+    creating_root = true;
+    nwos_write_object_to_disk(&private_root_obj_ref, &root_obj, sizeof(root_obj));
+    creating_root = false;
+
+    nwos_terminate_objectify();
+}
+#endif
+
+
+bool nwos_set_root_object(ObjRef* ref)
 {
     C_struct_Root root_obj;
     ObjRef verify_ref;
     C_struct_Class_Definition class_def;
 
-    nwos_read_object_from_disk(ref, &root_obj, sizeof(root_obj));
+    assert(!nwos_ready);
 
-    if (nwos_reference_type(ref) == Public_Reference)
+    if (nwos_read_object_from_disk(ref, &root_obj, sizeof(root_obj)))
     {
-	assert(nwos_private_path == NULL);   /* only allow if no private objects are in play */
+	if (nwos_reference_type(ref) == Public_Reference)
+	{
+	    assert(nwos_private_path == NULL);   /* only allow if no private objects are in play */
 
-	copy_reference(&nwos_public_class_definition_class_ref, &root_obj.class_definition_class);
-    }
-    else
-    {
-	assert(nwos_private_path != NULL);   /* only allow if private objects are in play */
+	    copy_reference(&nwos_public_class_definition_class_ref, &root_obj.class_definition_class);
+	}
+	else
+	{
+	    assert(nwos_private_path != NULL);   /* only allow if private objects are in play */
 
-	copy_reference(&nwos_private_class_definition_class_ref, &root_obj.class_definition_class);
+	    copy_reference(&nwos_private_class_definition_class_ref, &root_obj.class_definition_class);
 
-	nwos_read_class_definition(&nwos_private_class_definition_class_ref, &class_def);
+	    nwos_read_class_definition(&nwos_private_class_definition_class_ref, &class_def);
 
-	copy_reference(&nwos_public_class_definition_class_ref, &class_def.header.object.clone_of);
+	    copy_reference(&nwos_public_class_definition_class_ref, &class_def.header.object.clone_of);
+	}
+
+	copy_reference(&nwos_reference_list_class_ref, &root_obj.reference_list_class);
+
+	assert(nwos_find_public_class_definition("REFERENCE LIST", &verify_ref));
+	assert(is_same_object(&verify_ref, &nwos_reference_list_class_ref));
+
+	nwos_ready = true;
     }
 
-    copy_reference(&nwos_reference_list_class_ref, &root_obj.reference_list_class);
-
-    assert(nwos_find_public_class_definition("REFERENCE LIST", &verify_ref));
-    assert(is_same_object(&verify_ref, &nwos_reference_list_class_ref));
-
-    nwos_ready = true;
+    return nwos_ready;
 }
 
 
@@ -1383,35 +1543,25 @@ void nwos_generate_new_id(ObjRef* ref)
 /* Verification routines */
 /*************************/
 
+#if 0
 static void check_common_header(ObjRef* ref, CommonHeader* common)
 {
   /* could check time stamp is valid here?  anything else? */
 }
+#endif
 
 
-/* ref is only needed for error messages */
-
-static void check_object_header(ObjRef* ref, ObjectHeader* obj_header, uint8 stored[4])
+static bool object_header_checksum_matches(ObjectHeader* obj_header, uint8 stored[4])
 {
     uint8 computed[4];
-    bool checksum_ok;
 
     nwos_crc32_calculate((uint8*)obj_header, sizeof(*obj_header), computed);
 
-    checksum_ok = (stored[0] == computed[0] || stored[1] == computed[1] || stored[2] == computed[2] || stored[3] == computed[3]);
-
-    if (!checksum_ok)
-    {
-	printf("object: %02x%02x%02x%02x - bad header checksum, ", ref->id[0], ref->id[1], ref->id[2], ref->id[3]);
-	printf("size %zd - ", sizeof(*obj_header));
-	printf("computed %02x%02x%02x%02x - ", computed[0], computed[1], computed[2], computed[3]);
-	printf("stored %02x%02x%02x%02x\n", stored[0], stored[1], stored[2], stored[3]);
-	assert(checksum_ok);
-    }
+    return (stored[0] == computed[0] && stored[1] == computed[1] && stored[2] == computed[2] && stored[3] == computed[3]);
 }
 
 
-static void check_object_data(ObjRef* ref, uint8* object, size_t total_size, uint8 stored[4])
+static bool check_object_data(ObjRef* ref, uint8* object, size_t total_size, uint8 stored[4])
 {
     uint8 computed[4];
     size_t data_size;
@@ -1434,24 +1584,7 @@ static void check_object_data(ObjRef* ref, uint8* object, size_t total_size, uin
 
     nwos_crc32_calculate((uint8*)object + sizeof(EveryObject), data_size, computed);
 
-    if (stored[0] != computed[0] || stored[1] != computed[1] || stored[2] != computed[2] || stored[3] != computed[3])
-    {
-	printf("object: %02x%02x%02x%02x - bad data checksum, ", ref->id[0], ref->id[1], ref->id[2], ref->id[3]);
-	printf("size %zd - ", data_size);
-	printf("computed %02x%02x%02x%02x - ", computed[0], computed[1], computed[2], computed[3]);
-	printf("stored %02x%02x%02x%02x\n", stored[0], stored[1], stored[2], stored[3]);
-
-	printf("\n");
-	{
-	  int i;
-	  for (i = 0; i < sizeof(EveryObject) + data_size; i++)
-	  {
-	      printf("%02x%c", object[i], (i % 16) == 15 ? '\n' : ' ');
-	  }
-	  printf("\n");
-	}
-	exit(1);
-    }
+    return memcmp(computed, stored, sizeof(computed)) == 0;
 }
 
 
@@ -1460,41 +1593,11 @@ static void check_object_data(ObjRef* ref, uint8* object, size_t total_size, uin
 /* Routines to read blocks, objects, etc */
 /*****************************************/
 
-#ifdef DEBUG_ENCRYPTION
-uint8 save_unencrypted[FILE_BLOCK_SIZE];
-uint8 save_add_random[FILE_BLOCK_SIZE];
-uint8 save_mixed_up[FILE_BLOCK_SIZE];
-uint8 save_encrypted[FILE_BLOCK_SIZE];
-#endif
-
-
-#ifdef DEBUG_ENCRYPTION
-void compare_bufs(uint8 buf1[FILE_BLOCK_SIZE], uint8 buf2[FILE_BLOCK_SIZE], char* descr)
+bool nwos_read_object_from_disk_and_old_decrypt(ObjRef* ref, void* object, size_t size, uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
 {
-  int i;
-  for (i = 8; i < FILE_BLOCK_SIZE; i++) if (buf1[i] != buf2[i]) break;
-  if (i == FILE_BLOCK_SIZE)
-    {
-      printf("%s - OK\n", descr);
-    }
-  else
-    {
-      printf("%s - bad\n", descr);
-      for (i = 0; i < FILE_BLOCK_SIZE; i++)
-	{
-	  printf("%02x-%02x%c", buf1[i], buf2[i], i%16==15 ? '\n' : ' ');
-	}
-    }
-}
-#endif
-
-
-bool nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t size, uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
-{
-    uint8 buffer2[FILE_BLOCK_SIZE];
+    uint8 encrypted[FILE_BLOCK_SIZE];
 #ifndef PUBLIC_MODE
-    uint8 buffer1[FILE_BLOCK_SIZE];
-    uint8* ptrobj = (uint8*)object;
+    uint8 scrambled[FILE_BLOCK_SIZE];
     int i;
     int j;
 #endif
@@ -1502,15 +1605,12 @@ bool nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t si
 
     assert(!is_void_reference(ref));
 
-    if (nwos_read_block(ref, buffer2))
+    if (nwos_read_block(ref, encrypted))
     {
-	assert(block_contains_object(buffer2, ref));
+	assert(block_contains_object(encrypted, ref));
 #ifdef PUBLIC_MODE
-	memcpy(object, buffer2, size);
+	memcpy(object, encrypted, size);
 #else
-#ifdef DEBUG_ENCRYPTION
-	compare_bufs(buffer2, save_encrypted, "Encrypted");
-#endif
 
 #ifdef DISABLE_SECURITY_FEATURES
 	if (true)
@@ -1518,41 +1618,81 @@ bool nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t si
 	if (nwos_reference_type(ref) == Public_Reference)
 #endif
 	{
-	    memcpy(object, buffer2, size);    /* just copy the buffer into the object */
+	    memcpy(object, encrypted, size);    /* just copy the buffer into the object */
 	}
 	else
 	{
 	    /* copy the first 8 bytes directly, not encrypted */
-	    memcpy(buffer1, buffer2, 8);
+	    memcpy(object, encrypted, 8);
 
 	    /* decrypt the remaining bytes */
-	    BF_cbc_encrypt((buffer2 + 8), (buffer1 + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_DECRYPT);
-
-#if 0
-	    printf("ivec: ");
-	    for (i = 0; i < 8; i++) printf("%02x ", ivec[i]);
-	    printf("\n");
-#endif
-
-#ifdef DEBUG_ENCRYPTION
-	    compare_bufs(buffer1, save_mixed_up, "Mixed up");
-#endif
+	    BF_cbc_encrypt((encrypted + 8), (scrambled + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_DECRYPT);
 
 	    assert(size <= FILE_BLOCK_SIZE);
 
-	    /* copy the first 8 bytes directly accross */
-	    for (i = 0; i < 8; i++) ptrobj[i] = buffer1[i];
-
 	    for (i = 8; i < size; i++)
 	    {
 		j = seq_table[i];    /* find where the next byte is stored */
-		ptrobj[i] = buffer1[j];    /* get this byte from there */
+		((uint8*)object)[i] = scrambled[j];    /* get this byte from there */
 	    }
 	}
 
-#ifdef DEBUG_ENCRYPTION
-	compare_bufs(ptrobj, save_unencrypted, "Unencrypted");
 #endif
+	result = true;
+    }
+
+    return result;
+}
+
+
+bool nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t size, uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
+{
+    uint8 scrambled[FILE_BLOCK_SIZE];
+#ifndef PUBLIC_MODE
+    uint8 encrypted[FILE_BLOCK_SIZE];
+    uint8 decrypted[FILE_BLOCK_SIZE];
+    int i;
+    int j;
+#endif
+    bool result = false;
+
+    assert(!is_void_reference(ref));
+
+    if (nwos_read_block(ref, scrambled))
+    {
+	assert(block_contains_object(scrambled, ref));
+#ifdef PUBLIC_MODE
+	memcpy(object, scrambled, size);
+#else
+
+#ifdef DISABLE_SECURITY_FEATURES
+	if (true)
+#else
+	if (nwos_reference_type(ref) == Public_Reference)
+#endif
+	{
+	    memcpy(object, scrambled, size);    /* just copy the buffer into the object */
+	}
+	else
+	{
+	    /* copy the first 8 bytes directly, not encrypted */
+	    memcpy(object, scrambled, 8);
+
+	    /* descramble */
+	    for (i = 8; i < FILE_BLOCK_SIZE; i++)
+	    {
+		j = seq_table[i];    /* find where the next byte is stored */
+		encrypted[i] = scrambled[j];    /* get this byte from there */
+	    }
+
+	    assert(size <= FILE_BLOCK_SIZE);
+
+	    /* decrypt */
+	    BF_cbc_encrypt((encrypted + 8), (decrypted + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_DECRYPT);
+
+	    memcpy((object + 8), (decrypted + 8), (size - 8));
+	}
+
 #endif
 	result = true;
     }
@@ -1717,7 +1857,7 @@ typedef struct
 } Extra_Data_Block;
 
 
-void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
+bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 {
     EveryObject* header;
     uint8 ivec[IVEC_SIZE];
@@ -1727,6 +1867,7 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
     size_t bytes_remaining;
     int i;
     int seq = 0;
+    bool use_old_decryption = false;
 
     assert(!is_void_reference(ref));
 
@@ -1745,10 +1886,27 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
     ptr_obj = object;
 
-    assert(nwos_read_object_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq]));
+    if (!nwos_read_object_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
 
     assert(sizeof(ObjRef) == 4);
 
+    header = (EveryObject*) &extra;
+
+    /* check_common_header(ref, &header->common);     -- this doesn't do anything right now */
+
+    if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
+    {
+	memset(ivec, 0, sizeof(ivec));
+	if (!nwos_read_object_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+
+	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
+	{
+	    return false;
+	}
+
+	use_old_decryption = true;
+    }
+
     *ptr_obj++ = extra.flags[0];
     *ptr_obj++ = extra.flags[1];
     *ptr_obj++ = extra.flags[2];
@@ -1758,6 +1916,7 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
     *ptr_obj++ = extra.id.id[2];
     *ptr_obj++ = extra.id.id[3];
 
+
     while (bytes_remaining > FILE_BLOCK_SIZE)
     {
 	for (i = 0; i < SIZE_EXTRA_BLOCK_DATA; i++)
@@ -1771,7 +1930,15 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
 	seq++;
 
-	assert(nwos_read_object_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq]));
+	if (use_old_decryption)
+	{
+	    if (!nwos_read_object_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	}
+	else
+	{
+	    if (!nwos_read_object_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	}
+
     }
 
     assert(bytes_remaining <= FILE_BLOCK_SIZE);
@@ -1790,33 +1957,40 @@ void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size)
 
     header = (EveryObject*) object;
 
-    check_common_header(ref, &header->common);
-
-    check_object_header(ref, &header->object, header->common.header_chksum);
-
 #ifndef USE_PREDEFINED_STRUCTS
 #ifndef PUBLIC_MODE
     /* check to see if this is a file object (which has an older revision) */
     if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, get_file_001_reference()))
     {
-	check_object_data(ref, (uint8*)object, sizeof(C_struct_File_001), header->common.data_chksum);
-	nwos_update_file_001_object_to_current(object, size);
+	if (check_object_data(ref, (uint8*)object, sizeof(C_struct_File_001), header->common.data_chksum))
+	{
+	    nwos_update_file_001_object_to_current(object, size);
+	    return true;
+	}
     }
     else if (nwos_reference_type(ref) == Private_Reference && is_same_object(&header->common.class_definition, get_file_002_reference()))
     {
-	check_object_data(ref, (uint8*)object, sizeof(C_struct_File_002), header->common.data_chksum);
-	nwos_update_file_002_object_to_current(object, size);
+	if (check_object_data(ref, (uint8*)object, sizeof(C_struct_File_002), header->common.data_chksum))
+	{
+	    nwos_update_file_002_object_to_current(object, size);
+	    return true;
+	}
     }
     else
 #endif
 #endif
     {
-	check_object_data(ref, (uint8*)object, size, header->common.data_chksum);
+	if (check_object_data(ref, (uint8*)object, size, header->common.data_chksum))
+	{
+	    return true;
+	}
     }
+
+    return false;
 }
 
 
-void nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t max_size, size_t (*size_function)(void*))
+bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t max_size, size_t (*size_function)(void*))
 {
     EveryObject* header;
     uint8 ivec[IVEC_SIZE];
@@ -1828,6 +2002,7 @@ void nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
     int seq = 0;
     size_t xyzzy;
     char msg[128];
+    bool use_old_decryption = false;
 
     assert(!is_void_reference(ref));
 
@@ -1842,10 +2017,27 @@ void nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
 
     ptr_obj = object;
 
-    assert(nwos_read_object_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq]));
+    if (!nwos_read_object_from_disk_and_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
 
     assert(sizeof(ObjRef) == 4);
 
+    header = (EveryObject*) &extra;
+
+    /* check_common_header(ref, &header->common);     -- this doesn't do anything right now */
+
+    if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
+    {
+	memset(ivec, 0, sizeof(ivec));
+	if (!nwos_read_object_from_disk_and_old_decrypt(ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+
+	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
+	{
+	    return false;
+	}
+
+	use_old_decryption = true;
+    }
+
     *ptr_obj++ = extra.flags[0];
     *ptr_obj++ = extra.flags[1];
     *ptr_obj++ = extra.flags[2];
@@ -1881,7 +2073,15 @@ void nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
 
 	seq++;
 
-	assert(nwos_read_object_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq]));
+
+	if (use_old_decryption)
+	{
+	    if (!nwos_read_object_from_disk_and_old_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	}
+	else
+	{
+	    if (!nwos_read_object_from_disk_and_decrypt(&next_ref, &extra, sizeof(extra), ivec, nwos_random_sequence[seq])) return false;
+	}
     }
 
     assert(bytes_remaining <= FILE_BLOCK_SIZE);
@@ -1900,11 +2100,7 @@ void nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* object, size_t
 
     header = (EveryObject*) object;
 
-    check_common_header(ref, &header->common);
-
-    check_object_header(ref, &header->object, header->common.header_chksum);
-
-    check_object_data(ref, (uint8*)object, (*size_function)(object), header->common.data_chksum);
+    return check_object_data(ref, (uint8*)object, (*size_function)(object), header->common.data_chksum);
 }
 
 
@@ -1917,6 +2113,23 @@ void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header)
     memset(ivec, 0, sizeof(ivec));
     assert(nwos_read_object_from_disk_and_decrypt(ref, header, sizeof(*header), ivec, nwos_random_sequence[0]));
 
+    /* check_common_header(ref, &header->common); -- this currently doesn't do anything */
+
+    if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* could be old encryption */
+    {
+	memset(ivec, 0, sizeof(ivec));
+	assert(nwos_read_object_from_disk_and_old_decrypt(ref, header, sizeof(*header), ivec, nwos_random_sequence[0]));
+
+	if (!object_header_checksum_matches(&header->object, header->common.header_chksum))   /* neither decryption worked */
+	{
+	    printf("\n");
+	    fflush(stdout);
+	    fprintf(stderr, "Unable to decrypt object headers: %08x\n", nwos_ref_to_word(ref));
+	    nwos_terminate_objectify();
+	    exit(1);
+	}
+    }
+
 #if 0
     printf("read header path: %s\n", path);
     printf("read header pointer: %p\n", header);
@@ -1926,10 +2139,6 @@ void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header)
       printf("\n");
     }
 #endif
-
-    check_common_header(ref, &header->common);
-
-    check_object_header(ref, &header->object, header->common.header_chksum);
 }
 
 
@@ -1939,10 +2148,10 @@ void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header)
 
 void nwos_write_object_to_disk_and_encrypt(ObjRef* ref, void* object, size_t size, uint8 ivec[IVEC_SIZE], uint8 seq_table[FILE_BLOCK_SIZE])
 {
-    uint8 buffer2[FILE_BLOCK_SIZE];
+    uint8 scrambled[FILE_BLOCK_SIZE];
 #ifndef PUBLIC_MODE
-    uint8 buffer1[FILE_BLOCK_SIZE];
-    uint8* ptrobj = (uint8*)object;
+    uint8 decrypted[FILE_BLOCK_SIZE];
+    uint8 encrypted[FILE_BLOCK_SIZE];
     int i;
     int j;
 #endif
@@ -1950,13 +2159,9 @@ void nwos_write_object_to_disk_and_encrypt(ObjRef* ref, void* object, size_t siz
     assert(!is_void_reference(ref));
     assert(size <= FILE_BLOCK_SIZE);
 
-#ifdef DEBUG_ENCRYPTION
-    memcpy(save_unencrypted, object, FILE_BLOCK_SIZE);
-#endif
-
 #ifdef PUBLIC_MODE
-    memset(buffer2, 0, FILE_BLOCK_SIZE);
-    memcpy(buffer2, object, size);
+    memset(scrambled, 0, FILE_BLOCK_SIZE);
+    memcpy(scrambled, object, size);
 #else
 
 #ifdef DISABLE_SECURITY_FEATURES    /* always do the unencryped code */
@@ -1965,44 +2170,32 @@ void nwos_write_object_to_disk_and_encrypt(ObjRef* ref, void* object, size_t siz
     if (nwos_reference_type(ref) == Public_Reference)
 #endif
     {
-	memset(buffer2, 0, FILE_BLOCK_SIZE);    /* just copy the object into the buffer */
-	memcpy(buffer2, object, size);
+	memset(scrambled, 0, FILE_BLOCK_SIZE);    /* just copy the object into the buffer */
+	memcpy(scrambled, object, size);
     }
     else
     {
-	/* copy the first 8 bytes directly accross */
-	for (i = 0; i < 8; i++) buffer1[i] = ptrobj[i];
+	/* copy the first 8 bytes straight across */
+	memcpy(scrambled, object, 8);
+
+	memcpy((decrypted + 8), (object + 8), size - 8);
 
 	/* fill any empty slots with random junk */
-	if (size < FILE_BLOCK_SIZE)
-	{
-	    for (i = 8; i < FILE_BLOCK_SIZE; i++) buffer1[i] = random();
-	}
+	for (i = size; i < FILE_BLOCK_SIZE; i++) decrypted[i] = random();
 
-	for (i = 8; i < size; i++)
+	/* and encrypt the remainder */
+	BF_cbc_encrypt((decrypted + 8), (encrypted + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_ENCRYPT);
+
+	for (i = 8; i < FILE_BLOCK_SIZE; i++)
 	{
 	    /* find where the next byte is stored */
 	    j = seq_table[i];    /* find where the next byte is stored */
-	    buffer1[j] = ptrobj[i];    /* save this byte there */
+	    scrambled[j] = encrypted[i];    /* save this byte there */
 	}
-
-#ifdef DEBUG_ENCRYPTION
-	memcpy(save_mixed_up, buffer1, FILE_BLOCK_SIZE);
-#endif
-
-	/* copy the first 8 bytes straight across */
-	memcpy(buffer2, buffer1, 8);
-
-	/* and encrypt the remainder 8 bytes at a time */
-	BF_cbc_encrypt((buffer1 + 8), (buffer2 + 8), (FILE_BLOCK_SIZE - 8), &blowfish_key, ivec, BF_ENCRYPT);
     }
 #endif
 
-#ifdef DEBUG_ENCRYPTION
-    memcpy(save_encrypted, buffer2, FILE_BLOCK_SIZE);
-#endif
-
-    nwos_write_block(ref, buffer2);
+    nwos_write_block(ref, scrambled);
 }
 
 
@@ -2134,13 +2327,29 @@ void nwos_get_object_class_without_update(ObjRef* obj, ObjRef* object_class)
 
     assert(nwos_read_object_from_disk_and_decrypt(obj, &header, sizeof(header), ivec, nwos_random_sequence[0]));
 
-    check_common_header(obj, &header.common);
+    /* check_common_header(obj, &header.common);  -- this doesn't currently do anything */
 
     /* if it is not a reference list check the header checksum */
 
     if (!is_same_object(&header.common.class_definition, &nwos_reference_list_class_ref))
     {
-	check_object_header(obj, &header.object, header.common.header_chksum);
+	if (!object_header_checksum_matches(&header.object, header.common.header_chksum))   /* could be old encryption */
+	{
+	    memset(ivec, 0, sizeof(ivec));
+	    assert(nwos_read_object_from_disk_and_old_decrypt(obj, &header, sizeof(header), ivec, nwos_random_sequence[0]));
+
+	    if (!is_same_object(&header.common.class_definition, &nwos_reference_list_class_ref))
+	    {
+		if (!object_header_checksum_matches(&header.object, header.common.header_chksum))   /* neither decryption worked */
+		{
+		    printf("\n");
+		    fflush(stdout);
+		    fprintf(stderr, "Unable to decrypt object class without update: %08x\n", nwos_ref_to_word(obj));
+		    nwos_terminate_objectify();
+		    exit(1);
+		}
+	    }
+	}
     }
 
     copy_reference(object_class, &header.common.class_definition);
