@@ -27,6 +27,24 @@
 --
 --
 -- $Log: objectify.h,v $
+-- Revision 1.151  2008/09/01 03:00:09  jsedwards
+-- Change nwos_initialize_objectify declaration so private_root_object_ref is
+-- no longer passed back and nwos_set_root_object returns success or failure.
+--
+-- Revision 1.150  2008/08/31 10:46:14  jsedwards
+-- Changed nwos_read_variable_sized_object_from_disk to return bool instead of
+-- void.
+--
+-- Revision 1.149  2008/08/31 10:42:35  jsedwards
+-- Changed nwos_read_object_from_disk to return bool instead of void.
+--
+-- Revision 1.148  2008/08/30 12:50:27  jsedwards
+-- Corrected previous log entry (accidentally checked in with incorrect message).
+--
+-- Revision 1.147  2008/08/30 12:46:15  jsedwards
+-- Changed nwos_initialize_objectify parameters, no longer pass key parameters
+-- instead it returns the private root object reference.
+--
 -- Revision 1.146  2008/08/03 15:34:30  jsedwards
 -- Added DISC_LIST_* flags and masks and changed nwos_read_files_disc_list to
 -- return uint32 instead of int.
@@ -935,9 +953,9 @@ extern void nwos_log_sha1sum(const char* file_name, uint8 sha1_digest[SHA1_DIGES
 extern void nwos_log_sha256sum(const char* file_name, uint8 sha256_digest[SHA256_DIGEST_SIZE]);
 extern void nwos_log_sha512sum(const char* file_name, uint8 sha512_digest[SHA512_DIGEST_SIZE]);
 
-extern void nwos_initialize_objectify(uint8 bf_key[16], uint32 linear, uint32 serial, AccessType type, const char* path);
+extern void nwos_initialize_objectify(AccessType type, const char* path);
 extern void nwos_terminate_objectify();
-extern void nwos_set_root_object(ObjRef* ref);
+extern bool nwos_set_root_object(ObjRef* ref);
 extern void nwos_flush_bit_maps();
 extern bool nwos_in_public_mode();
 
@@ -971,8 +989,8 @@ extern void nwos_create_reference_list(ObjRef* for_obj, ObjRef* ref_list);
 extern void nwos_create_object(uint64 id, EveryObject* header);
 extern void nwos_read_class_definition(ObjRef* ref, C_struct_Class_Definition* class_def);
 extern void nwos_read_object_headers_from_disk(ObjRef* ref, EveryObject* header);
-extern void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size);
-extern void nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* obj, size_t max_size, size_t (*size_function)(void*));
+extern bool nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size);
+extern bool nwos_read_variable_sized_object_from_disk(ObjRef* ref, void* obj, size_t max_size, size_t (*size_function)(void*));
 extern void nwos_read_reference_list_from_disk(ObjRef* ref, ReferenceList* object, size_t size);
 
 
