@@ -27,6 +27,13 @@
 --
 --
 -- $Log: objectify_private.h,v $
+-- Revision 1.71  2008/09/01 18:16:29  jsedwards
+-- Changed nwos_create_root to not take any parameters.
+--
+-- Revision 1.70  2008/08/31 18:11:49  jsedwards
+-- Remove duplicate declaration of nwos_read_object_from_disk which is
+-- declared in objectify.h.
+--
 -- Revision 1.69  2008/07/31 14:01:56  jsedwards
 -- Added external declaration for nwos_read_object_from_disk_and_old_decrypt.
 --
@@ -475,13 +482,12 @@ extern void nwos_write_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE]);
 extern void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size);
 extern void nwos_write_public_object_to_disk(ObjRef* ref, void* object, size_t size);
 extern void nwos_overwrite_object_to_disk(ObjRef* ref, void* object, size_t size);
-extern void nwos_read_object_from_disk(ObjRef* ref, void* object, size_t size);
 extern void nwos_create_english_language(ObjRef* ref);
 extern void nwos_create_all_twelve_months();
 extern void nwos_create_all_area_codes();
 extern void nwos_set_spelling_class_ref(ObjRef* spelling_class_ref);
 extern void nwos_set_name_class_ref(ObjRef* name_class_ref);
-extern bool nwos_create_root(ObjRef* ref);
+extern void nwos_create_root(void);
 extern void nwos_remove_object(ObjRef* ref);
 extern void nwos_create_reference_list_with_existing_id(ObjRef* for_obj, ObjRef* ref_list);
 extern void nwos_flush_dirty_ref_lists();
