@@ -27,6 +27,18 @@
 --
 --
 -- $Log: prep_disk.c,v $
+-- Revision 1.60  2008/09/01 18:24:14  jsedwards
+-- Moved almost all of the code that prepared to create the root object into
+-- the nwos_create_root function in objectify.c.
+--
+-- Revision 1.59  2008/08/31 17:58:26  jsedwards
+-- Add assert around call to nwos_read_object_from_disk because it now returns
+-- false when it fails, whereas before it would assert itself.
+--
+-- Revision 1.58  2008/08/30 12:46:16  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
 -- Revision 1.57  2008/07/19 15:40:20  jsedwards
 -- Removed the #ifndef TEST_PASS_PHRASE around the verifying it that it is
 -- okay to write over the storage.  Too dangerous!!
@@ -345,14 +357,13 @@ void print_usage(char* name, bool specify_private_size)
 
     if (specify_private_size == false)
     {
-	fprintf(stderr, "usage: %s [--allocate-all] [--no-private]\n\n", name);
+	fprintf(stderr, "usage: %s [--no-private]\n\n", name);
     }
     else
     {
-	fprintf(stderr, "usage: %s [--allocate-all] [--no-private] size\n\n", name);
+	fprintf(stderr, "usage: %s [--no-private] size\n\n", name);
     }
 
-    fprintf(stderr, "  --allocate-all: allocate all the chunks in storage.\n");
     fprintf(stderr, "  --no-private: don't create the initial private blocks (if importing old data)\n");
 
     if (specify_private_size == true)
@@ -561,32 +572,12 @@ uint32 write_chunks(int fd, uint8* chunk, uint32 num_chunks)
 }
 
 
-/* this is the same as the normal routine in objectify.c except it doesn't translate the reference for the disk */
-
-static void set_public_root_object(ObjRef* ref)
-{
-    C_struct_Root root_obj;
-    ObjRef verify_ref;
-
-    nwos_read_object_from_disk(ref, &root_obj, sizeof(root_obj));
-    copy_reference(&nwos_public_class_definition_class_ref, &root_obj.class_definition_class);
-    copy_reference(&nwos_reference_list_class_ref, &root_obj.reference_list_class);
-
-    assert(nwos_find_public_class_definition("REFERENCE LIST", &verify_ref));
-    assert(is_same_object(&verify_ref, &nwos_reference_list_class_ref));
-}
-
-
 /***************************************************************************/
 /* Program to prepare a disk or file for NWOS storage.                     */
 /* Now there is a bit map of used blocks as the first block in every chunk */
 /* 8192 * 8 = 65536 blocks * 256 bytes per block = 16777216 bytes in chunk */
 /***************************************************************************/
 
-/* This function is a temporary hack to allow all chunks to be allocated up front */
-extern void nwos_allocate_all_chunks_hack(uint32 ref, void (*progress_report)(float));
-
-
 int main(int argc, char* argv[])
 {
     int obj_fd;
@@ -597,7 +588,6 @@ int main(int argc, char* argv[])
     int arg_index = 1;
     bool is_file = false;
     bool do_private_blocks = true;
-    bool allocate_all = false;
 
     uint32 total_blocks;
     uint32 reserved_private_blocks;
@@ -605,14 +595,6 @@ int main(int argc, char* argv[])
     uint32 blocks_written;
     uint32 num_chunks;
     Disk_Header disk_header;
-    ObjRef root_obj_ref;
-    ObjRef private_root_obj_ref;
-    ObjRef public_root_class_ref;
-    /* ObjRef private_root_class_ref; */
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
     char msg[128];
     char* p;
 #if 0
@@ -663,10 +645,6 @@ int main(int argc, char* argv[])
 	{
 	    do_private_blocks = false;
 	}
-	else if (strcmp(argv[arg_index], "--allocate-all") == 0)
-	{
-	    allocate_all = true;
-	}
 	else
 	{
 	    fprintf(stderr, "\nunknown option: %s\n\n", argv[arg_index]);
@@ -887,73 +865,7 @@ int main(int argc, char* argv[])
 #ifndef TEST_PASS_PHRASE
 	understand_security();   /* make sure he or she understands the risks */
 #endif
-
-	nwos_get_key_from_password(big_key, sizeof(big_key));
-
-	memcpy(bf_key, big_key, 16);
-	linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-	memcpy(private_root_obj_ref.id, big_key+20, 4);
-	serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-	nwos_initialize_objectify(bf_key, linear, serial, READ_WRITE, DEFAULT_FILE);
-
-
-	if (allocate_all)
-	{
-	    nwos_start_progress_bar();
-	    nwos_allocate_all_chunks_hack(nwos_ref_to_word(&private_root_obj_ref), &nwos_update_progress_bar);
-	    nwos_finish_progress_bar();
-	}
-	else
-	{
-	    nwos_allocate_new_chunk(nwos_ref_to_word(&private_root_obj_ref));
-	}
-
-	root_obj_ref.id[0] = 0;
-	root_obj_ref.id[1] = 0;
-	root_obj_ref.id[2] = 0;
-	root_obj_ref.id[3] = 1;
-
-	set_public_root_object(&root_obj_ref);
-
-	assert(nwos_find_public_class_definition("ROOT", &public_root_class_ref));
-
-
-	/* create the private root object */
-
-	nwos_set_security_level(Security_Extreme);
-
-	assert(nwos_check_blocks_available(3));   /* this should always succeed */
-
-	printf("private root object id: %02x%02x%02x%02x\n", 
-	       private_root_obj_ref.id[0],
-	       private_root_obj_ref.id[1],
-	       private_root_obj_ref.id[2],
-	       private_root_obj_ref.id[3]);
-
-	nwos_clone_class_definition(&nwos_public_class_definition_class_ref, &nwos_private_class_definition_class_ref);
-
-#if 0
-	/* currently I cannot think of a reason to have a private root class because there isn't a list of root objects */
-
-	printf("private class definition id: %02x%02x%02x%02x\n", 
-	       nwos_private_class_definition_class_ref.id[0],
-	       nwos_private_class_definition_class_ref.id[1],
-	       nwos_private_class_definition_class_ref.id[2],
-	       nwos_private_class_definition_class_ref.id[3]);
-
-	nwos_clone_class_definition(&public_root_class_ref, &private_root_class_ref);
-
-	printf("private root class definition id: %02x%02x%02x%02x\n", 
-	       private_root_class_ref.id[0],
-	       private_root_class_ref.id[1],
-	       private_root_class_ref.id[2],
-	       private_root_class_ref.id[3]);
-#endif
-
-	assert(nwos_create_root(&private_root_obj_ref));
-
-	nwos_terminate_objectify();
+	nwos_create_root();
     }
 
     return 0;
