@@ -27,6 +27,18 @@
 --
 --
 -- $Log: verify_file.c,v $
+-- Revision 1.24  2008/09/01 18:53:48  jsedwards
+-- Fixed Bug #2043130 - asserts if given an absolute path.
+--
+-- Revision 1.23  2008/09/01 03:13:45  jsedwards
+-- Change for new nwos_initialize_objectify calling convention (doesn't pass
+-- back root_object_reference anymore) and removed call to nwos_set_root_object
+-- because initialize calls it now.
+--
+-- Revision 1.22  2008/08/30 12:46:20  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
 -- Revision 1.21  2008/07/11 14:03:56  jsedwards
 -- Fix Feature Request #1996867 "Move verify_file --shred option warning
 -- before doing it".  Instead of moving the warning I removed the warning
@@ -409,11 +421,6 @@ void process_directory(const char* path, MatchCode match, Remove_Option remove_o
 
 int main(int argc, char* argv[])
 {
-    ObjRef root_object_ref;
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
     int i;
     Remove_Option remove_option = Do_Not_Remove;
     bool do_public = false;
@@ -502,27 +509,11 @@ int main(int argc, char* argv[])
 
     if (do_public)
     {
-	nwos_initialize_objectify(NULL, 0, 0, PUBLIC, NULL);
-
-	root_object_ref.id[0] = 0;
-	root_object_ref.id[1] = 0;
-	root_object_ref.id[2] = 0;
-	root_object_ref.id[3] = 1;
-
-	nwos_set_root_object(&root_object_ref);
+	nwos_initialize_objectify(PUBLIC, NULL);
     }
     else
     {
-	nwos_get_key_from_password(big_key, sizeof(big_key));
-
-	memcpy(bf_key, big_key, 16);
-	linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-	memcpy(root_object_ref.id, big_key+20, 4);
-	serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-	nwos_initialize_objectify(bf_key, linear, serial, READ_ONLY, path);
-
-	nwos_set_root_object(&root_object_ref);
+	nwos_initialize_objectify(READ_ONLY, path);
     }
 
     for (/* use i as left by loop above */; i < argc; i++)
@@ -540,7 +531,11 @@ int main(int argc, char* argv[])
 	}
 	else if (strlen(argv[i]) + 1 > sizeof(normalized_path))
 	{
-	    fprintf(stderr, "ERROR: the path '%s' is too long for the buffer (exceeds %d characters), skipped\n", argv[i], sizeof(normalized_path));
+	    fprintf(stderr, "WARNING: the path '%s' is too long for the buffer (exceeds %d characters) - ignored.\n", argv[i], sizeof(normalized_path));
+	}
+	else if (*argv[i] == '/')
+	{
+	    fprintf(stderr, "WARNING: the path '%s' is absolute and this version cannot process it - ignored.\n", argv[i]);
 	}
 	else
 	{
